require 'android_app_index/version'
require 'algoliasearch'
require 'nokogiri'
require 'open-uri'

# Note that we use the singleton instance of the Algolia client.
# To use the instance version, we use Algolia::Client.new(application_id, api_key)
# and pass that Client to Algolia::Index. This is an interesting decision for
# the algolia api to make. It makes a simple example very easy but actually
# removes any opinion about how the api should be used. It seems easy enough for
# users to create a singleton if they want so this seems like a poor API choice on
# their part.
Algolia.init application_id: ENV['ANDROID_APP_INDEX_ALGOLIA_APP_ID'],
             api_key: ENV['ANDROID_APP_INDEX_ALGOLIA_API_KEY']

class PlayStoreParser
  private def get_page(package)
    Nokogiri::HTML(open("https://play.google.com/store/apps/details?id=#{package}&hl=en"))
  end

  private def get_icon_url(page)
    images = page.search('img.cover-image').map do |node|
      { alt: node.attribute('alt').value, url: "https:#{node.attribute('src').value}" }
    end

    images.select { |image| image[:alt] == 'Cover art' }[0][:url]
  end

  # TODO: Figure out how to properly handle charset.
  private def replace_dash(name)
    name.gsub('&acirc;&#128;&#147;', '-')
  end

  private def get_name(page)
    replace_dash page.search('div.id-app-title').first.inner_html
  end

  def parse_play_store(package)
    page = get_page package

    {
      icon_url: get_icon_url(page),
      name: get_name(page),
      package: package,
    }
  end
end


module AndroidAppIndex
  @play_store_parser = PlayStoreParser.new
  @index = Algolia::Index.new('android_apps')

  @hit_to_result = lambda { |hit|
    {
      name: hit['name'],
      package: hit['package'],
      icon_url: hit['icon_url']
    }
  }

  @index_and_parse_play_store = lambda { |package, name, icon_url|
    app_data = {
      package: package,
      icon_url: icon_url,
      name: name
    }

    needs_details = name.nil? || icon_url.nil?
    app_data = @play_store_parser.parse_play_store(package) if needs_details

    app_data[:objectID] = package
    @index.add_object(app_data)
    app_data
  }

  def self.get(package)
    @index.get_object(package)
  rescue Algolia::AlgoliaProtocolError
    # Handle the error that occurs if the app is not yet indexed.
    index package
  end

  def self.index(package, name = nil, icon_url = nil)
    @index_and_parse_play_store.call(package, name, icon_url)
  rescue OpenURI::HTTPError
    {
      error: 'no_app_found',
      error_message: "Unable to find an app with package=#{package}"
    }
  end

  def self.search(query)
    search_results = @index.search(
      query,
      attributesToRetrieve: 'package,icon_url,name',
      hitsPerPage: 100
    )

    search_results['hits'].map(&@hit_to_result)
  end
end
